/*
 * File         : BV_Application.C
 * Author       : Tomasz Stachowiak
 * Created      : Fri Jul 24 17:19:28 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <strings.h>

#include <map.h>

#include "BV_Member.h"
#include "BV_Session_Control.h"
#include "BV.h"
#include "UserData.h"
#include "BV_AppReadThread.h"
#include "BV_Application.h"

BV_Application::BV_Application(int port)
{
  data_sock_ = -1;
  sess_ctrl = BV_Session_Control::instance();
  port_ = port;
  read_thread_ = new BV_AppReadThread(this);
  sequence = 0;
}

BV_Application::~BV_Application()
{
  sess_ctrl->destroy();
  delete read_thread_;
}

int BV_Application::init()
{
  struct sockaddr_in addr;

  if ((data_sock_ = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      return -1;
    }
  
  bzero(&addr, sizeof(struct sockaddr_in));
  addr.sin_family      = AF_INET;
  addr.sin_addr.s_addr = htonl(INADDR_ANY);
  addr.sin_port        = htons(port_);
  
  if(bind(data_sock_, &addr, sizeof(struct sockaddr_in)) < 0)
    {
      printf("ERROR: app socket bind\n");
      fflush(stdout);
      return -1;
    }
  
  if(read_thread_->start() < 0)
    {
      printf("ERROR: read thread start\n");
      fflush(stdout);
      return -1;
    }

  return 0;
}

void BV_Application::close()
{
  //read_thread_->stop();
  read_thread_->kill();
  ::close(data_sock_);
}

int BV_Application::send_data(int type, char *data, int len)
{
  char *ncsdata;
  char *tmp;
  int size;
  int sent = 0;


  map <int, BV_Member> *members = sess_ctrl->get_member_list();
  //printf("sd0.1 %d\n", members);
  //fflush(stdout);
  map <int, BV_Member>::iterator i = members->begin();
  //printf("sd0.2\n");
  //fflush(stdout);

  size = sizeof(RTPheader) + len;
  ncsdata = new char[size];
  fill_rtp_header((RTPheader *)ncsdata, type);
  tmp = ncsdata + sizeof(RTPheader);
  memcpy(tmp, data, len); 

  ((RTPheader *)ncsdata)->ssrc = sess_ctrl->get_local_id();

  while(i != members->end())
    {
      if((*i).second.get_id() != sess_ctrl->get_local_id())
	{
	  //BVN_DEBUG("BV_Application: data sent to user %d, size %d\n", i, size);
	  if(send_packet(data_sock_, (*i).second.get_host(), port_, ncsdata, size) < 0)
	    return -1;
	}
      i++;
    }

  delete ncsdata;

  return 0;
}

int BV_Application::send_packet(int sock, int hostid, int port, char *msg, int len)
{
  struct sockaddr_in addr;

  bzero(&addr, sizeof(struct sockaddr_in));
  addr.sin_family      = AF_INET;
  addr.sin_addr.s_addr = htonl(hostid);
  addr.sin_port        = htons(port);

  if(sendto(sock, msg, len, 
         0, &addr, sizeof(struct sockaddr_in)) < 0)
    return -1;  

  return 0;
}

void BV_Application::fill_rtp_header(RTPheader *header, unsigned int payload)
{
  header->type = 2;
  header->p = 0;
  header->x = 0;
  header->cc = 0;
  header->m = 0;
  header->pt = payload;
  header->seq = sequence++;
  header->ts = get_time_stamp();
  header->ssrc = 0;  
}

unsigned int BV_Application::get_time_stamp()
{
  unsigned int result;
  struct timeval time;

  gettimeofday(&time);
  result = (time.tv_sec - SEP_18_97)*1000 + (unsigned int)(time.tv_usec/1000);
  
  return result;
}

int BV_Application::get_data_sock()
{
  return data_sock_;
}

int BV_Application::get_internal_id(int id)
{
  BV_Member *mbr = sess_ctrl->get_member(id);
  if(mbr == NULL)
    return -1;

  return mbr->get_internal_id();
}

